m_per_oz = 34.7e-6
mu0 = math.pi * 4e-7 # H / m
ur = 1
omega = 2 * math.pi * 13.56e6
tq = 35.
resistivity = 1.72e-8 # ohm-m
temp_co = 3.862e-3 # K^-1

def l_ant_square(layout, turns, din, dout, thick_mm):
    k1, k2 = dict(
        square = (2.34, 2.75),
        hexagonal = (2.33, 3.82),
        octagonal = (2.25, 3.55),
        )[layout]

    din /= 1000.
    dout /= 1000.
    thick = thick_mm / 1000.
    d = (dout + din) / 2.
    p = (dout - din) / (dout + din)
    return k1 * mu0 * (turns ** 2) * d / (1 + k2 * p)

def l_ant_circle(turns, r, wire_diam):
    r /= 1000.
    wire_diam /= 1000.
    return mu0 * (turns ** 1.9) * r * math.log(r / wire_diam)

def l_ant_spiral(turns, d, thick_mm):
    d /= 1000.
    thick = thick_mm / 1000.

    return 31.33 * mu0 * (turns ** 2) * d / (8 * d + 11 * thick)

def r_copper_trace(w, h, l, temp = 25):
    resistivity = 1.72e-8 # ohm-m
    temp_co = 3.862e-3 # K^-1
    area = w * h / 1e6
    return resistivity * l / 1e3 / area * (1 + (temp_co * (temp - 25)))

def r_ant_rect(w, h, ct, cw, spacing, turns):
    r = 0
    for i in range(turns):
        r += r_copper_trace(ct, cw, (w + h) * 2)
        w -= 2 * (cw + spacing)
        h -= 2 * (cw + spacing)
    return r

def len_ant_rect(w, h, ct, cw, spacing, turns):
    length = 0
    for i in range(turns):
        length += (w + h) * 2
        w -= 2 * (cw + spacing)
        h -= 2 * (cw + spacing)
    return length

def ant_rect(w, h, ct, cw, spacing, turns, layers = 1):
    L = len_ant_rect(w, h, ct, cw, spacing, turns) * layers
    l = l_ant_rect(w, h, ct, cw, spacing, turns) * layers
    r = r_ant_rect(w, h, ct, cw, spacing, turns) * layers
    q = omega * l / r
    rq = 0.
    q2 = q

#    if not .3e-6 < l < 3e-6:
#        return

    if not tq * .9 < q < tq * 1.1:
        rq = .5 * (omega * l / 35. - r)

#        print rq

        if rq < 1:
            return

        rq = e24_closest(rq)
        q2 = (omega * l / (r + rq * 2))

#        print e24, rq, q2

        if not tq * .9 < q2 < tq * 1.1:
            return

    return (w, h), (ct, cw, spacing), (turns, layers), (L, l, r, q, rq, q2)
